@@ -1,6 +1,6 @@
 #ifndef FILTERS_H_INCLUDED
 #define FILTERS_H_INCLUDED
-#define FILTERS_H_VERSION "$Id: filters.h,v 1.43 2011/09/04 11:10:56 fabiankeil Exp $"
+#define FILTERS_H_VERSION "$Id: filters.h,v 1.46 2013/12/24 13:32:51 fabiankeil Exp $"
 /*********************************************************************
  *
  * File        :  $Source: /cvsroot/ijbswa/current/filters.h,v $
@@ -42,20 +42,6 @@
 
 #include "project.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
-struct access_control_addr;
-struct client_state;
-struct http_request;
-struct http_response;
-struct current_action_spec;
-struct url_actions;
-struct url_spec;
-
-
 /*
  * ACL checking
  */
@@ -92,6 +78,11 @@ extern void get_url_actions(struct client_state *csp,
 extern void apply_url_actions(struct current_action_spec *action,
                               struct http_request *http,
                               struct url_actions *b);
+
+extern struct re_filterfile_spec *get_filter(const struct client_state *csp,
+                                             const char *requested_name,
+                                             enum filter_type requested_type);
+
 /*
  * Determining parent proxies
  */
@@ -123,10 +114,6 @@ extern struct http_response *direct_response(struct client_state *csp);
 extern const char filters_rcs[];
 extern const char filters_h_rcs[];
 
-#ifdef __cplusplus
-} /* extern "C" */
-#endif
-
 #endif /* ndef FILTERS_H_INCLUDED */
 
 /*
