@@ -7,7 +7,7 @@
 # A regression test "framework" for Privoxy. For documentation see:
 # perldoc privoxy-regression-test.pl
 #
-# $Id: privoxy-regression-test.pl,v 1.91 2013/03/07 14:10:04 fabiankeil Exp $
+# $Id: privoxy-regression-test.pl,v 1.93 2013/12/24 13:36:58 fabiankeil Exp $
 #
 # Wish list:
 #
@@ -491,8 +491,7 @@ sub mark_matching_tests_for_skipping($) {
                 my $message = sprintf("Marking test %s for ignoring. Overwrite condition: %s.",
                                       $regression_tests[$s][$r]{'number'}, $overwrite_condition);
 
-                # XXX: Should eventually be downgraded to LL_FILE_LOADING.
-                log_message($message);
+                l(LL_FILE_LOADING, $message);
 
                 # XXX: Should eventually get its own key so get_skip_reason()
                 #      can tell about the overwrite condition.
@@ -717,12 +716,12 @@ sub execute_regression_tests () {
             #      from different sections isn't possible.
             #      Is this worth changing the layout?
             fisher_yates_shuffle(\@regression_tests);
-            for (my $s = 0;  $s < @regression_tests; $s++) {
+            for (my $s = 0; $s < @regression_tests; $s++) {
                 fisher_yates_shuffle($regression_tests[$s]);
             }
         }
 
-        for (my $s = 0;  $s < @regression_tests; $s++) {
+        for (my $s = 0; $s < @regression_tests; $s++) {
 
             my $r = 0;
 
