@@ -52,7 +52,7 @@ Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 =cut
 
-# $Id: docbook2man-spec.pl,v 1.4 2006/07/18 14:49:14 david__schmidt Exp $
+# $Id: docbook2man-spec.pl,v 1.5 2015/08/12 10:33:56 fabiankeil Exp $
 
 use SGMLS;			# Use the SGMLS package.
 use SGMLS::Output;		# Use stack-based output.
@@ -259,11 +259,8 @@ sgml('</REFMETA>', sub {
 	push_output('file', "$manpage_title.$manpage_sect");
 
 	output <<_END_BANNER;
-.\\" This manpage has been automatically generated by docbook2man 
-.\\" from a DocBook document.  This tool can be found at:
-.\\" <http://shell.ipoline.com/~elmert/comp/docbook2X/> 
-.\\" Please send any bug reports, improvements, comments, patches, 
-.\\" etc. to Steve Cheng <steve\@ggi-project.org>.
+.\\" This manpage has been generated by docbook2man-spec.pl
+.\\" (included in the Privoxy source tarball) from a DocBook document.
 _END_BANNER
 
 	my $manpage_date = `date "+%d %B %Y"`;
